/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_LOSS_FUNC_H
#define _GCTL_LOSS_FUNC_H

// library's head files
#include "gctl/core.h"

namespace gctl
{
    class loss_func
    {
    public:
        loss_func();
        loss_func(const array<double> &tar, norm_type_e n_type);
        virtual ~loss_func();

        void init(const array<double> &tar, norm_type_e n_type);
        void set_uncertainty(double uncer);
        void set_uncertainty(const array<double> &uncer);
        double evaluate(const array<double> &x, array<double> &g);

    private:
        bool init_;
        unsigned int tnum_;
        norm_type_e ntype_;
        array<double> tars_, diff_;
        array<double> us_;
    };
}

#endif // _GCTL_LOSS_FUNC_H